/***************************************************************************
 *   Copyright (C) 2009 by QEVEN Team                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 **************************************************************************/

/*
 * req_manager.cpp
 *
 *  Created on: Oct 17, 2008
 *      Author: wyxknouth
 */

#include <QFileInfo>
#include <QMessageBox>
#include <QFileDialog>
#include <QSettings>


#include "req_manager.h"

req_manager::req_manager(QString MEncoder, QString MPlayer, QObject *parent ) :
      QObject( parent )
{
   bmencoder  = MEncoder;
   bmplayer = MPlayer;

   readSettings();

   iSplash = new QPixmap( ":main/Image/Splash.png" );
   m_Splash = new QSplashScreen( *iSplash );
   wait = false;
   reloj = new QTimer(this);
   connect(reloj,SIGNAL(timeout()),this,SLOT(stopWait()));
}

req_manager::~req_manager()
{
   delete iSplash;
   delete m_Splash;
   delete reloj;
}

/**
 * Funcion requerida para mostrar el Splash en pantalla
 * Verifica la existencia del MEncoder
 */
bool  req_manager::show(QApplication& app)
{
   m_Splash->show();
   m_Splash->showMessage( QObject::tr( "Verificando aplicaciones externas..." ),
                          Qt::AlignBottom);

   // QTest::qWait( 1000 );

   this->Wait(1000,app);

   //app.processEvents();

   m_Splash->showMessage( QObject::tr( "Verificando MEncoder..." ), Qt::AlignBottom );

   this->Wait(500,app);
   if( mencoder() )
   {
      m_Splash->showMessage( QObject::tr( "MEncoder                       (OK)" ),
                             Qt::AlignBottom);

      this->Wait(500,app);
   }
   else
   {
      QMessageBox* nMEnc = new  QMessageBox();
      nMEnc->setStandardButtons( QMessageBox::Ok | QMessageBox::Cancel );
      nMEnc->setWindowTitle( QObject::tr( "Faltan algunos programas necesarios" ) );
      nMEnc->setText( QObject::tr( "QEVEN no ha podido encontrar el programa  MEncoder en su sistema, indique la ruta manualmente o instalelo mas tarde." ) );
      QString directory;
      switch( nMEnc->exec() )
      {
            case  QMessageBox::Ok :
         directory = QFileDialog::getOpenFileName( 0,
                                                   QObject::tr( "Ubicacion de MEncoder" ),
                                                   QDir::rootPath() );
         if( !directory.isEmpty() )
         {
            bmencoder = QDir::toNativeSeparators( directory );

            writeSettings();
         }
         else
         {
            return false;
         }
         break;
            case QMessageBox::Cancel :
         return false;
         break;
            default:
         break;
      }
   }

   //app.processEvents();
   m_Splash->showMessage( QObject::tr( "Verificando MPlayer..." ), Qt::AlignBottom );
   this->Wait(500,app);
   if( mplayer() )
   {
      m_Splash->showMessage( QObject::tr( "MPlayer                       (OK)" ),
                             Qt::AlignBottom);
      this->Wait(500,app);
   }
   else
   {
      QMessageBox* nMPla = new  QMessageBox();
      nMPla->setStandardButtons( QMessageBox::Ok | QMessageBox::Cancel );
      nMPla->setWindowTitle( QObject::tr( "Faltan algunos programas necesarios" ) );
      nMPla->setText( QObject::tr( "QEVEN no ha podido encontrar el programa  MPlayer en su sistema, indique la ruta manualmente o instalelo mas tarde." ) );
      QString directory;
      switch( nMPla->exec() )
      {
            case QMessageBox::Ok :
         directory = QFileDialog::getOpenFileName( 0,
                                                   QObject::tr( "Ubicacion de MPlayer" ), QDir::rootPath() );

         if ( !directory.isEmpty() )
         {
            bmplayer = QDir::toNativeSeparators( directory );

            writeSettings();
         }
         else
         {
            return false;
         }
         break;
            case  QMessageBox::Cancel :
         return false;
         break;
            default:
         break;
      }
   }

   this->Wait(1000,app);
   return true;
}

/**
 * Funcion para terminar de mostrar el splash y dar paso a la ventana principal
 */
void req_manager::finish( QMainWindow& mw )
{
   m_Splash->finish( &mw );
   emit splash_finished();
}

bool req_manager::mencoder()
{
   QFileInfo* menc = new QFileInfo( bmencoder );
   {
      return menc->exists();
   }
}


bool req_manager::mplayer()
{
   QFileInfo* menc = new QFileInfo( bmplayer );
   {
      return menc->exists();
   }
}

/**
 * Evento. Funcion que se ejecuta al cerrar la ventana
 */
void req_manager::closeEvent( QCloseEvent * /*event*/ )
{
   writeSettings();
}

/**
 * Funcion para guardar las configuraciones del sistema
 *
 */
void req_manager::writeSettings()
{
#ifdef Q_OS_LINUX
   QSettings settings(  QDir::homePath() + "/.QEVEN/config.ini", QSettings::IniFormat );
#else
   QSettings ini(QSettings::IniFormat,QSettings::UserScope,
                 QCoreApplication::organizationName(),
                 QCoreApplication::applicationName());
   QString dir = QFileInfo(ini.fileName()).absolutePath()+"/QEVEN/config.ini";
   QSettings settings( dir,  QSettings::IniFormat );
#endif
   settings.setValue( "MEncoder", bmencoder );
   settings.setValue( "MPlayer", bmplayer );
}

/**
 * Funcion para leer las configuraciones del sistema
 */
void req_manager::readSettings()
{
#ifdef Q_OS_LINUX
   QSettings settings(  QDir::homePath() + "/.QEVEN/config.ini", QSettings::IniFormat );
#else
   QSettings ini(QSettings::IniFormat,QSettings::UserScope,
                 QCoreApplication::organizationName(),
                 QCoreApplication::applicationName());
   QString dir = QFileInfo(ini.fileName()).absolutePath()+"/QEVEN/config.ini";
   QSettings settings( dir,  QSettings::IniFormat );
#endif
   bmencoder = settings.value( "MEncoder", bmencoder ).toString();
   bmplayer = settings.value( "MPlayer", bmplayer ).toString();
}

void req_manager::stopWait()
{
   wait = false;
}

void req_manager::Wait(int interval,QApplication& app)
{
   wait = true;
   reloj->setInterval(interval);
   reloj->start();
   while(wait)
   {
      app.processEvents();
   }
   reloj->stop();
}
