/***************************************************************************
 *   Copyright (C) 2009 by QEVEN Team                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 **************************************************************************/

#include "video_manager.h"

#include <QMessageBox>
#include <QSettings>

namespace video_manager {

   Video_Manager::Video_Manager()
   {
      m_video_list = new QList<Video_Item*>();
      m_format = vcd;
      MPlayer = "";
   }

   Video_Manager::Video_Manager( e_video_format format )
   {
      m_video_list = 0;
      m_video_list = new QList<Video_Item*>();
      m_format = format;
   }

   Video_Manager::~Video_Manager()
   {
      delete m_video_list;
   }

   void Video_Manager::add_Video( QString file )
   {
      Video_Analyzer analyzer( file ,MPlayer);
      QString filename = analyzer.get_Video_Name();
      QString file_dir = analyzer.get_Video_Directory();
      int duration = analyzer.get_Duration();
      int videobitrate = analyzer.get_Video_Bitrate();
      int audiobitrate = analyzer.get_Audio_Bitrate();
      int video_width = 0;
      int video_height = 0;
      int video_width_original = analyzer.get_Video_Width();
      int video_height_original = analyzer.get_Video_Height();
      int aid = analyzer.get_aid();
      float fps = analyzer.get_Fps();

      QStringList lstr = file.split(".");
      lstr.removeLast();
      QString subtsrt = lstr.join(".") + ".srt";
      QString subtSrt = lstr.join(".") + ".Srt";
      QString subtsub = lstr.join(".") + ".sub";
      QString subtSub = lstr.join(".") + ".Sub";

      bool has_sub = false;
      QString subPath = "";
      QString font = "";
#ifdef Q_OS_LINUX
      font = "./Fonts/FreeSans.ttf";
#endif
#ifdef Q_OS_WIN32
      font = ".\\Fonts\\FreeSans.ttf";
#endif
      QSettings settings( "QEVEN", "QEVEN" );
      int scale = settings.value( "S_scale", 3 ).toInt();

      if(QFile::exists(subtsrt))
      {
         has_sub = true;
         subPath  = subtsrt;
      }
      else
      {
         if(QFile::exists(subtsub))
         {
            has_sub = true;
            subPath = subtsub;
         }
         else
         {
            if(QFile::exists(subtSrt))
            {
               has_sub = true;
               subPath  = subtSrt;
            }
            else
            {
               if(QFile::exists(subtSub))
               {
                  has_sub = true;
                  subPath = subtSub;
               }
            }
         }
      }

      /*
        Esta serie de condiciones es para establecer
        un bitrate estandar para cada tipo de proyecto.
        Si se comentan entonces se crea el tipo de proyecto
        con el bitrate original del video.
        En el caso de los DIVX y XVID se toma
        el bitrate original del video.
        */

      if( m_format == vcd )
      {
         videobitrate = 1152;
         audiobitrate = 224;
         video_width = 352;
         video_height =240;
      } else

         if( m_format == flv )
         {
         videobitrate = 500;
         audiobitrate = 56;
      } else
         if(m_format == dvd)
         {
         videobitrate = 4000;
         audiobitrate = 224;
         video_width = 720;
         video_height = 480;
      } else
         if(m_format == svcd)
         {
         videobitrate = 1800;
         audiobitrate = 224;
         video_width = 480;
         video_height = 480;
      }
      else
      {
         video_width = video_width_original;
         video_height = video_height_original;
      }


      Video_Item *temp_video = new Video_Item( filename,
                                               file_dir,
                                               videobitrate,
                                               audiobitrate,
                                               duration,
                                               m_format,
                                               ":autoaspect",
                                               has_sub,
                                               subPath,
                                               font,
                                               scale);
      temp_video->set_Video_Width(video_width);
      temp_video->set_Video_Height(video_height);
      temp_video->set_Fps(fps);
      temp_video->set_Video_Width_Original(video_width_original);
      temp_video->set_Video_Height_Original(video_height_original);
      temp_video->set_aid(aid);
      m_video_list->append( temp_video );
   }

   void Video_Manager::add_Video( Video_Item* temp_video )
   {
      m_video_list->append(temp_video);
   }

   bool Video_Manager::is_Empty()
   {
      return m_video_list->isEmpty();
   }

   void Video_Manager::set_Format( e_video_format format )
   {
      m_format = format;
   }

   void Video_Manager::set_Format( QString format )
   {
      if( format == "vcd" )
      {
         m_format = vcd;
      }
      else
      {
         if( format == "svcd" )
         {
            m_format = svcd;
         }
         else
         {
            if ( format == "dvd" )
            {
               m_format = dvd;
            }
            else
            {
               if( format == "divx" )
               {
                  m_format = divx;
               }
               else
               {
                  if( format == "xvid" )
                  {
                     m_format = xvid;
                  }
                  else
                  {
                     if( format == "flv" )
                     {
                        m_format = flv;
                     }
                  }
               }
            }
         }
      }
   }

   void Video_Manager::delete_Video( int pos )
   {
      m_video_list->removeAt(pos);
   }

   Video_Item* Video_Manager::get_Video( int pos )
   {
      return m_video_list->value( pos );
   }

   int Video_Manager::get_Total_Size()
   {
      float total=0;
      for( int i=0; i< m_video_list->count(); i++)
      {
         total += m_video_list->value(i)->get_Video_Size();
      }

      return total;
   }

   int Video_Manager::get_Length()
   {
      return m_video_list->count();
   }

   /**
    * Funcion para cambiar las opciones a un elemento especifico de la lista
    * @param pos Posicion en la lista del elemento
    */
   void Video_Manager::change_Video_Settings ( int pos,
                                               QString vaspect,
                                               int vbitrate,
                                               int abitrate,
                                               bool has_subtitles,
                                               QString subtitle_file,
                                               QString subtitle_font,
                                               int subtitle_scale,
                                               int resolution,
                                               int width,
                                               int height)
   {
      m_video_list->value( pos )->set_Video_Aspect( vaspect );
      m_video_list->value( pos )->set_Video_Bitrate( vbitrate );
      m_video_list->value( pos )->set_Audio_Bitrate( abitrate );
      m_video_list->value( pos )->set_Has_Subtitles( has_subtitles );
      m_video_list->value( pos )->set_Subtitles_File( subtitle_file );
      m_video_list->value( pos )->set_Subtitles_Font( subtitle_font );
      m_video_list->value( pos )->set_Subtitles_Scale( subtitle_scale );
      m_video_list->value(pos)->set_Video_Width(width);
      m_video_list->value(pos)->set_Video_Height(height);
      m_video_list->value(pos)->set_Video_Resolution(resolution);
   }

   /**
    * Funcion para cambiar las opciones a todos los items de la lista
    */
   void  Video_Manager::change_vopts_all(  int vbitrate,
                                           int abitrate,
                                           QString vaspect,
                                           QString subtitle_font,
                                           int subtitle_scale,
                                           int resolution,
                                           int width,
                                           int height)
   {
      for(int i = 0;i < m_video_list->size();i++)
      {
         m_video_list->at(i)->Update(vbitrate,abitrate,vaspect,subtitle_font,subtitle_scale,30000/1001,resolution,width,height);
      }
   }

   QList<Video_Item*>* Video_Manager::get_Video_List()
   {
      return m_video_list;
   }

   /**
    * Funcion de acceso al formatode video de la lista
    */
   QString Video_Manager::format()
   {
      switch( m_format )
      {
      case vcd :
         return "vcd";
         break;
      case svcd :
         return "svcd";
         break;
      case dvd :
         return "dvd";
         break;
      case divx :
         return "divx";
         break;
      case xvid :
         return "xvid";
         break;
      case flv :
         return "flv";
         break;
      }

      return "";
   }

   e_video_format Video_Manager::getFormat()
   {
      return m_format;
   }

   /**
    * Funcion que me devuelve la lista de videos en formato xml
    */
   QString Video_Manager::toXML()
   {
      QString xml = "";
      for( int i = 0;i < m_video_list->count(); i++ )
      {
         xml += m_video_list->operator[](i)->toXML();
      }

      return xml;
   }

   void Video_Manager::setProgramDir(QString MPlayer)
   {
      this->MPlayer = MPlayer;
   }

}


