/***************************************************************************
 *   Copyright (C) 2009 by QEVEN Team                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 **************************************************************************/

#include "main_window.h"
#include "aboutus.h"
#include <QFileDialog>
#include <QMessageBox>
#include <QSettings>
#include <QDesktopWidget>
#include <QTextStream>
#include <QTextCodec>
#include <QFile>
#include <QDropEvent>
#include <QUrl>
#include "lenguage.h"

using namespace std;

bool the_warning = false;

Main_Window::Main_Window( QWidget* parent, Qt::WFlags fl ) :
      QMainWindow( parent, fl ),
      Ui::Main_Form()
{
   setupUi(this);

   flagClose = false; //flag para manejar la salida del programa
   projectFileName = "";
   this->MEncoder = "";
   this->MPlayer = "";
   visible_Window = 0;
   createMenu();

   listWidget->setViewMode( QListView::ListMode );
   listWidget->setIconSize( QSize( 96, 84 ) );

   wizard_gui = new Project_GUI( this );
   options_gui = new Options_GUI;
   converter_gui = new Converter_GUI;
   m_video_list = new Video_Manager();
   about = new AboutUs( this );
   about->setWindowModality( Qt::WindowModal );

   Disable_Buttons();
   readSettings();
   setAcceptDrops( true );

   listWidget->setViewMode( QListView::ListMode );
   listWidget->setIconSize( QSize( 96, 84 ) );

   setAcceptDrops( true );
   modified = false;

   //Se crea el system TrayIcon, se muestra y se conecta
   createTrayIcon();
   trayIcon->show();
   // inicializar el tryIcon de la ventana de conversion con el flag del primer uso
   this->converter_gui->set_try_icon( this->trayIcon, this->firstTrayMsg );

   createConnection();
}

Main_Window::~Main_Window()
{
   delete wizard_gui;
   delete options_gui;
   delete m_video_list;
   delete converter_gui;
   delete about;
}

/**
  Funcion para crear los menu de la aplicacion
  */
void Main_Window::createMenu()
{
   Language = new QActionGroup(this);
   QStringList lenguages = findQmFiles();
   cantLeng = lenguages.size();
   for(int i = 0 ; i < cantLeng; i++)
   {
       QString l = languageName(lenguages.at(i));
        QAction* temp = new QAction(l,this);
        temp->setCheckable(true);
        Language->addAction(temp);
        connect(temp,SIGNAL(triggered()),this,SLOT(ChangeLanguage()));
        menuLenguaje->addAction(temp);
   }
   Language->setExclusive(true);
}

QStringList Main_Window::findQmFiles()
{

#ifdef Q_OS_LINUX
   QDir dir("./translation");
   if(qApp->applicationDirPath() == "/usr/bin")
      dir = "/usr/share/qeven/translation";
#endif
#ifdef Q_OS_WIN32
   QDir dir = QDir(".\\translation\\");
#endif

   QStringList fileNames = dir.entryList(QStringList("*.qm"), QDir::Files,
                                         QDir::Name);
   QMutableStringListIterator i(fileNames);
   while (i.hasNext()) {
      i.next();
      i.setValue(dir.filePath(i.value()));
   }
   return fileNames;
}

#if QT_VERSION >= 0x040500
QString Main_Window::languageName(const QString &qmFile)
 {
     QTranslator translator;
     translator.load(qmFile);

     return translator.translate("Ui", "English");
 }
#else
QString Main_Window::languageName(const QString &qmFile)
 {
     QTranslator translator;
     translator.load(qmFile);

     return translator.translate("MainWindow", "English","",0);
 }

#endif


/**
  * Funcion para crear las conexiones
  */
void Main_Window::createConnection()
{
   connect( addButton, SIGNAL( clicked() ),
            this, SLOT( Add_Video() ) );
   connect( removeButton, SIGNAL( clicked() ),
            this, SLOT( Remove_Video() ) );
   connect( diskComboBox, SIGNAL( currentIndexChanged( int ) ),
            this, SLOT( Disk_Changed() ) );
   connect( startButton, SIGNAL( clicked() ),
            this, SLOT( Start_Conversion() ) );
   connect( optionsButton, SIGNAL( clicked() ),
            this, SLOT( Show_Options() ) );
   connect( options_gui,SIGNAL( signal_options_changed( const int&,
                                                        const int&,
                                                        const int&,
                                                        const QString&,
                                                        const bool&,
                                                        const QString&,
                                                        const QString&,
                                                        const int&,
                                                        const int&,
                                                        const int&,
                                                        const int&) ),
            this, SLOT( Read_Options_Changed( int,
                                              int,
                                              int,
                                              QString,
                                              bool,
                                              QString,
                                              QString,
                                              int,
                                              int,
                                              int,
                                              int) ) );
   connect( options_gui,SIGNAL( signal_options_applyall ( const int&,
                                                          const int&,
                                                          const QString&,
                                                          const QString&,
                                                          const int&,
                                                          const int&,
                                                          const int&,
                                                          const int&) ),
            this, SLOT( ApplyAll_Options_Changed( int,
                                                  int,
                                                  QString,
                                                  QString,
                                                  int,
                                                  int,
                                                  int,
                                                  int) ) );
   connect( wizard_gui, SIGNAL( signal_selected_project( const e_video_format&) ),
            this, SLOT( Project_Selected( const e_video_format& ) ) );
   connect( exitButton, SIGNAL( clicked() ),
            this, SLOT( Exit() ) );
   connect( exitProgram_action, SIGNAL( triggered() ),
            this, SLOT( Exit() ) );
   connect( newProject_action, SIGNAL( triggered() ),
            this, SLOT( New() ) );
   connect( saveProject_action, SIGNAL( triggered() ),
            this, SLOT( Save() ) );
   connect( openProject_action, SIGNAL( triggered() ),
            this, SLOT( Load() ) );
   connect( aboutQt_action, SIGNAL( triggered() ),
            qApp, SLOT( aboutQt() ) );
   connect( aboutUs_action, SIGNAL( triggered() ),
            about, SLOT( show() ) );
   connect( converter_gui, SIGNAL( Cancelled() ),
            this, SLOT( Show_Main_Window() ) );
   connect( converter_gui, SIGNAL( Finished() ),
            this, SLOT( Show_Main_Window() ) );
   connect(importFont_action,SIGNAL(triggered() ),
           this,SLOT(ImportFont()));
   connect(options_gui,SIGNAL(importarFont(QString)),
           this,SLOT(add_Font(QString)));
//   connect(LSpanish,SIGNAL(triggered()),this,SLOT(ChangeLanguage()));
//   connect(LEnglish,SIGNAL(triggered()),this,SLOT(ChangeLanguage()));
   connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
           this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
}
//---- SLOTS -----------------------------------
void Main_Window::Main_Window::Add_Video()
{
   m_video_list->setProgramDir(MPlayer);
   QStringList directory = QFileDialog::getOpenFileNames( this,
                                                          tr( "Agregar Archivo :" ),
                                                          this->lastProjectDir,
                                                          /*
        -Adicionadas extensiones ogg y ogv. on 2008-11-18 12:23:58.
        -Adicionadas y organizadas nuevas extensiones soportadas por mencoder. Lunes, 24 de Noviembre de 2008.*/

                                                          tr( "Todos los Archivos Soportados(*.mpg *.mpeg *.vob *.mp4 *.dat *.avi *.divx *.xvid *.wmv *.asf *.ogg *.ogv *.ogm *.mov *.qt *.rmvb *.rm *.flv *.mkv *.mka, *.m2t);;Videos MPEG (*.mpg *.mpeg *.vob *.mp4 *.dat, *.m2t);;Videos AVI(*.avi *.divx *.xvid);;Videos WMV(*.wmv *.asf);;Videos Ogg Vorbis(*.ogg *.ogv *.ogm);;Videos QuickTime (*.mov *.qt);;Videos Real Media (*.rmvb *.rm);;Videos Adobe Flash (*.flv);;Videos Matroska (*.mkv *.mka)"));


   if ( !directory.isEmpty() )
   {
      for( int i = 0; i < directory.count(); i++ )
      {

         m_video_list->add_Video( directory.at( i ) );

         Update_Space_Bar();
         Update_List();

         modified = true;
#ifdef Q_OS_LINUX
         QStringList path_aux = directory.at(i).split( "/" );
         path_aux.removeLast();
         lastProjectDir= path_aux.join( "/" ).append( "/" );
#endif

#ifdef Q_OS_WIN32
         QStringList path_aux = directory.at(i).split( "/" );
         path_aux.removeLast();
         lastProjectDir= path_aux.join( "\\" ).append( "\\" );
#endif
      }

   }
}

void Main_Window::Remove_Video()
{
   if( listWidget->currentRow() >= 0 )
   {
      m_video_list->delete_Video( listWidget->currentRow() );

      Update_Space_Bar();
      Update_List();

      modified = true;
   }
}

void Main_Window::About()
{
}

void Main_Window::Disk_Changed()
{
   Update_Space_Bar();
}

/**
 * SLOT. Crea un proyecto de acuerdo al formato seleccionado
 * en el Wizard.
 */
void Main_Window::Project_Selected( const e_video_format& project )
{
   m_video_list->set_Format( project );
   QString pro = "";
   switch(project)
   {
   case vcd: pro = "VCD - ";
      break;
   case svcd: pro = "SVCD - ";
      break;
   case dvd: pro = "DVD - ";
      break;
   case divx: pro = "DivX - ";
      break;
   case xvid: pro = "XviD - ";
      break;
   case flv: pro = "FLV - ";
      break;
   default: pro = "";

   }
   this->setWindowTitle(pro + "QEVEN");
   Enable_Buttons();
}

void Main_Window::Show_Options()
{
   if ( listWidget->currentRow() != -1 )
   {
      // objeto temporal para no repetir tantas veces al metodo get_Video...
      Video_Item* tmp = m_video_list->get_Video( listWidget->currentRow() );
      QString filename = tmp->get_Video_Name();
      QString fileDir  = tmp->get_Video_Path();
      int bitrate = tmp->get_Video_Bitrate();
      int abitrate = tmp->get_Audio_Bitrate();
      QString vaspect = tmp->get_Video_Aspect();
      bool has_subtitles = tmp->has_Subtitles();
      QString subtitles = tmp->get_Subtitles_File();
      QString font = tmp->get_Subtitles_Font();
      int scale = tmp->get_Subtitles_Scale();
      int resolution = tmp->get_Video_Resolution();
      int owidth = tmp->get_Video_Width_Original();
      int oheight = tmp->get_Video_Height_Original();

      options_gui->Initialize( fileDir,
                               filename,
                               listWidget->currentRow(),
                               bitrate,
                               abitrate,
                               vaspect,
                               has_subtitles,
                               subtitles,
                               font,
                               scale,
                               resolution,
                               owidth,
                               oheight);

      options_gui->show();
      options_gui->setWindowTitle(this->windowTitle() + tr(" | Opciones de Video"));
   }
   else
   {
      QMessageBox::information(
            this,
            tr("Seleccione un archivo."),
            tr("Debe seleccionar un archivo de la lista para editar sus opciones."),
            QMessageBox::Ok );
   }
}

void Main_Window::Read_Options_Changed( const int& item_selected,
                                        const int& vbitrate,
                                        const int& abitrate,
                                        const QString& vaspect,
                                        const bool& has_subtitles,
                                        const QString& sub_path,
                                        const QString& sub_font,
                                        const int& sub_scale,
                                        const int& resolution,
                                        const int& width,
                                        const int& height)
{
   m_video_list->change_Video_Settings( item_selected,
                                        vaspect,
                                        vbitrate,
                                        abitrate,
                                        has_subtitles,
                                        sub_path,
                                        sub_font,
                                        sub_scale,
                                        resolution,
                                        width,
                                        height);

   Update_Space_Bar();
}
/**
  *[SLOT] Funciona que informa a la lista el cambio de las opciones para todos los videos
  */
void Main_Window::ApplyAll_Options_Changed( const int& vbitrate,
                                            const int& abitrate,
                                            const QString& vaspect,
                                            const QString& sub_font,
                                            const int& sub_scale,
                                            const int& resolution,
                                            const int& width,
                                            const int& height)
{
   m_video_list->change_vopts_all(vbitrate,
                                  abitrate,
                                  vaspect,
                                  sub_font,
                                  sub_scale,
                                  resolution,
                                  width,
                                  height);
   Update_Space_Bar();
}

void Main_Window::Start_Conversion()
{
   //visible_Window = 1;
   if ( !m_video_list->is_Empty() )
   {
      QString output_dir = QFileDialog::getExistingDirectory( this,
                                                              tr("Seleccione el directorio de salida:"),
                                                              lastProjectDir );
      if(output_dir != "")
      {
         bool aux = var_preview->isChecked();
         converter_gui->SetProgramDir(MEncoder,MPlayer);
         converter_gui->Initialize( m_video_list->get_Video_List(),
                                    output_dir, aux);
         converter_gui->setWindowTitle(this->windowTitle() + tr(" | Convirtiendo ..."));
         // esto lo pase para aca pq realmente se debe poner en 1 cuando se este convirtiendo
         visible_Window = 1;
         // adicionado un toolTip para la aplicacion
         // hay trabajos pendientes.... comprobacion trivial
         if( 1 == this->visible_Window )
         {
            trayIcon->setToolTip(  tr( "QEVEN  Convirtiendo ..."));
         }

         this->hide();
         converter_gui->show();
      }
   }
   else
   {
      QMessageBox::information( this,
                                tr("Lista de Videos Vacia"),
                                tr("Debe agregar al menos un archivo a la lista de trabajos"),
                                QMessageBox::Ok );
   }
}

/**
 * SLOT. Termina la Aplicacion.
 */
void Main_Window::Exit()
{
   flagClose = true;

   // si la aplicacion se intenta terminar en pleno proceso a traves
   // del trayIcon entonces actualizamos el estado.
   try
   {
      this->converter_gui->Cancel();
   }
   catch(...)
   {
   }

   this->close();

}

/**
* SLOT. Cuando se crea un nuevo proyecto asegura que se limpie la lista
* de videos agregados, y manda a visualizar el Asistente de proyectos
*/
void Main_Window::New()
{
   if(maybeSave())
   {
      Reset();
      wizard_gui->show();
   }
}



/**
  * [SLOT] Funcion para importar nuevas fuentes para la aplicacion
  */
void Main_Window::ImportFont()
{
   QStringList directory = QFileDialog::getOpenFileNames( this,
                                                          tr( "Agregar Archivo :" ),
                                                          this->lastProjectDir,
                                                          tr( "Fuentes TrueType (*.ttf *.TTF)"));


   if ( !directory.isEmpty() )
   {
      for( int i = 0; i < directory.count(); i++ )
      {
         add_Font( directory.at( i ) );
         options_gui->fuentes();

#ifdef Q_OS_LINUX
         QStringList path_aux = directory.at(i).split( "/" );
         path_aux.removeLast();
         lastProjectDir= path_aux.join( "/" ).append( "/" );
#endif
#ifdef Q_OS_WIN32
         QStringList path_aux = directory.at(i).split( "/" );
         path_aux.removeLast();
         lastProjectDir= path_aux.join( "\\" ).append( "\\" );
#endif

      }
   }
}

void Main_Window::ChangeLanguage()
{
   //Por implementar todavia, incompleto
   QMessageBox::information(
         this,
         tr("QEVEN"),
         tr("Reinicie QEVEN para aplicar los cambios"),
         QMessageBox::Ok );

}

//------- FUNCTIONS -------------------------------
/**
 * Actualiza la barra de espacio ocupado
 * cada vez que se cambia la capacidad del disco
 * o cuando se agrega o elimina un video
 * de la lista.
 */
void Main_Window::Update_Space_Bar()
{
   float Disk_Capacity = 0;
   float Disk_Used = 0; //Aqui sumare la capacidad de cada
   // video en la lista
   // Si  0 entonces capacidad = 700 MB CD
   // Si  1 entonces capacidad = 4505.6 MB (4.4 GB) DVD
   if( diskComboBox->currentIndex() == 0 )
   {
      Disk_Capacity = 700;
   }
   else
   {
      Disk_Capacity = 4505.6;
   }

   Disk_Used = m_video_list->get_Total_Size();

   int Percent_Used = (int)(Disk_Used * 100 / Disk_Capacity);

   if( Percent_Used >= 100 )
   {
      spaceStatusBar->setMaximum( Percent_Used );
   }
   else
   {
      spaceStatusBar->setMaximum(100);
   }

   spaceStatusBar->setValue(Percent_Used);
}

/**
 * Funcion. Actualiza la lista de elementos
 * en el listWidget
 */
void Main_Window::Update_List()
{
   listWidget->clear();

   // usado para identificar ficheros "problematicos" y mostrar el mensaje adecuado
   bool warning = false;

   for (int i = 0; i < m_video_list->get_Length(); i++)
   {
      QListWidgetItem *previewButton = new QListWidgetItem( listWidget );
      previewButton->setIcon(QIcon( ":icons/Image/multimedia_icon.png" ) );
      previewButton->setText( m_video_list->get_Video( i )->get_Video_Name() );
      previewButton->setTextAlignment( Qt::AlignLeft );
      previewButton->setFlags( Qt::ItemIsSelectable | Qt::ItemIsEnabled );

      if( m_video_list->get_Video( i )->get_Video_Name().endsWith( ".wmv" , Qt::CaseInsensitive ) ||
          m_video_list->get_Video( i )->get_Video_Name().endsWith( ".rmvb" , Qt::CaseInsensitive ) ||
          m_video_list->get_Video( i )->get_Video_Name().endsWith( ".mov" , Qt::CaseInsensitive ) )
      {
         warning = true;
      }
   }

   // advertencia sobre los ficheros wmv, rmvb y mov que son problematicos
   if( false == the_warning && true == warning )
   {
      trayIcon->showMessage( tr( "Info" ), tr( "Los archivos con formato wmv, rmvb y mov pueden no convertirse correctamente." ) );

      the_warning = true;
   }
}

/**
 * Funcion. Habilita los botones de la interfaz principal
 */
void Main_Window::Enable_Buttons()
{
   addButton->setEnabled( true );
   removeButton->setEnabled( true );
   optionsButton->setEnabled( true );
   startButton->setEnabled( true );
}

/**
 * Funcion. Deshabilita los botones de la interfaz principal
 */
void Main_Window::Disable_Buttons()
{
   addButton->setEnabled( false );
   removeButton->setEnabled( false );
   optionsButton->setEnabled( false );
   startButton->setEnabled( false );
}

void Main_Window::Reset()
{
   m_video_list->get_Video_List()->clear();
   Update_List();
   Update_Space_Bar();
   projectFileName = "";
   modified = false;
}



/**
 * Funcion para guardar las configuraciones del sistema
 *
 */
void Main_Window::writeSettings()
{
#ifdef Q_OS_LINUX
   QSettings settings(  QDir::homePath() + "/.QEVEN/config.ini", QSettings::IniFormat );
#else
   QSettings ini(QSettings::IniFormat,QSettings::UserScope,
                 QCoreApplication::organizationName(),
                 QCoreApplication::applicationName());
   QString dir = QFileInfo(ini.fileName()).absolutePath()+"/QEVEN/config.ini";
   QSettings settings( dir,  QSettings::IniFormat );

#endif
   settings.setValue( "MW_Position", pos() );
   settings.setValue( "MW_Size", size() );
   settings.setValue( "LastProDir", lastProjectDir );
   settings.setValue( "DiskType", diskComboBox->currentIndex() );

    int lang = 0;
   for(int i = 0; i < cantLeng; i++)
   {
       if( menuLenguaje->actions().at(i)->isChecked())
       {
            lang = i;
            break;
        }
    }
   settings.setValue("language",lang);
}

/**
 * Funcion para leer las configuraciones del sistema
 */
void Main_Window::readSettings()
{
#ifdef Q_OS_LINUX
   QSettings settings(  QDir::homePath() + "/.QEVEN/config.ini", QSettings::IniFormat );
#else
   QSettings ini(QSettings::IniFormat,QSettings::UserScope,
                 QCoreApplication::organizationName(),
                 QCoreApplication::applicationName());
   QString dir = QFileInfo(ini.fileName()).absolutePath()+"/QEVEN/config.ini";
   QSettings settings( dir,  QSettings::IniFormat );
#endif
   QPoint pos = settings.value( "MW_Position", QPoint( 52, 47 ) ).toPoint();
   QSize size = settings.value( "MW_Size", QSize( 610, 601 ) ).toSize();
   move( pos );
   resize( size );
   diskComboBox->setCurrentIndex( settings.value( "DiskType", 0 ).toInt() );
   Disk_Changed();
   lastProjectDir = settings.value( "LastProDir", QDir::homePath() ).toString();

    int lang = settings.value( "language", 0 ).toInt();
   if(lang < cantLeng)
      menuLenguaje->actions().at(lang)->setChecked(true);
   else
   {
        if(cantLeng != 0)
            menuLenguaje->actions().at(0)->setChecked(true);
   }
}

/**
 * Funcion. Salva el proyecto que se encuentre abierto.
 */
void Main_Window::Save()
{
   QString fileName = QFileDialog::getSaveFileName( this,
                                                    QObject::tr( "Save As" ),
                                                    lastProjectDir,
                                                    QObject::tr( "QEVEN Project Files *.qvp" ) );

   if( !fileName.endsWith( ".qvp" ) )
   {
      fileName.append(".qvp");
   }

   Save( fileName );
}

void Main_Window::Save( QString fileName )
{
   if( fileName.isEmpty() )
   {
      return;
   }

   lastProjectDir = fileName;
   QFile file( fileName );

   if( !file.open( QFile::WriteOnly ) )
   {
      QMessageBox::warning( this,
                            tr("QEVEN"),
                            tr( "Cannot write file %1:\n%2." ).arg( fileName ).arg( file.errorString() ) );
      //		cout<<"couldn't open file "<<fileName.toStdString()<<" because "<<file.errorString().toStdString();
      //		cout<<endl;
      return;
   }

   QTextStream ts( &file );
   ts.setCodec( QTextCodec::codecForName( "UTF-8" ) );
   ts << "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n";
   ts << "<Project version=\"1.1\" format=\""+ m_video_list->format() + "\">\n";
   ts << m_video_list->toXML();
   ts << "</Project>\n";

   modified = false;
}

/**
 * Funcion. Carga un nuevo proyecto existente
 */
void Main_Window::Load()
{
   QString fileOpen = QFileDialog::getOpenFileName( this,
                                                    tr( "Open QEVEN Project File" ),
                                                    lastProjectDir,
                                                    tr( "QEVEN Project Files (*.qvp)" ) );

   if ( fileOpen.isEmpty() )
      return;

   QFile file( fileOpen );
   if( !file.open( QFile::ReadOnly | QFile::Text ) )
   {
      QMessageBox::warning( this,
                            tr( "QEVEN" ),
                            tr( "Cannot read file %1:\n%2." )
                            .arg( fileOpen )
                            .arg( file.errorString() ) );
      //       cout<<"couldn't open file "<<fileOpen.toStdString()<<" because "<<file.errorString().toStdString();
      //	       cout<<endl;
      return;
   }

   QDomDocument domDocument;
   QString errorStr;
   int errorLine;
   int errorColumn;
   if ( !domDocument.setContent( &file,
                                 true,
                                 &errorStr,
                                 &errorLine,
                                 &errorColumn ) )
   {
      QMessageBox::information( this,
                                QObject::tr( "QEVEN" ),
                                QObject::tr( "Parse error at line %1, column %2:\n%3" ).arg( errorLine ).arg( errorColumn ).arg( errorStr ) );
      return;
   }

   QDomElement root = domDocument.documentElement();
   if( root.tagName() != "Project" )
   {
      QMessageBox::information( this,
                                QObject::tr( "QEVEN" ),
                                QObject::tr( "Este no es un archivo de proyecto" ) );
      return;
   }
   else
   {
      if ( root.hasAttribute( "version" ) && root.attribute( "version" ) != "1.1" )
      {
         QMessageBox::information(this,
                                  QObject::tr("QEVEN"),
                                  QObject::tr( "EL fichero no pertenece a una version compatible " ) );
         return;
      }
   }

   Reset();
   projectFileName = file.fileName();
   m_video_list->set_Format(root.attribute( "format" ) );
   Project_Selected(m_video_list->getFormat());

   QDomElement child = root.firstChildElement();

   while ( !child.isNull() )
   {
      parseVideoElement( child );
      child = child.nextSiblingElement();
   }

   Update_List();
   Update_Space_Bar();
   Enable_Buttons();
}


/**
 * Funcion. Carga un nuevo proyecto existente
 */
void Main_Window::Load(QString fileO)
{
   //this->show();
   QString fileOpen = fileO;

   if ( fileOpen.isEmpty() )
      return;

   QFile file( fileOpen );
   if( !file.open( QFile::ReadOnly | QFile::Text ) )
   {
      QMessageBox::warning( this,
                            tr( "QEVEN" ),
                            tr( "Cannot read file %1:\n%2." )
                            .arg( fileOpen )
                            .arg( file.errorString() ) );
      //       cout<<"couldn't open file "<<fileOpen.toStdString()<<" because "<<file.errorString().toStdString();
      //	       cout<<endl;
      return;
   }

   QDomDocument domDocument;
   QString errorStr;
   int errorLine;
   int errorColumn;
   if ( !domDocument.setContent( &file,
                                 true,
                                 &errorStr,
                                 &errorLine,
                                 &errorColumn ) )
   {
      QMessageBox::information( this,
                                QObject::tr( "QEVEN" ),
                                QObject::tr( "Parse error at line %1, column %2:\n%3" ).arg( errorLine ).arg( errorColumn ).arg( errorStr ) );
      return;
   }

   QDomElement root = domDocument.documentElement();
   if( root.tagName() != "Project" )
   {
      QMessageBox::information( this,
                                QObject::tr( "QEVEN" ),
                                QObject::tr( "Este no es un archivo de proyecto" ) );
      return;
   }
   else
   {
      if ( root.hasAttribute( "version" ) && root.attribute( "version" ) != "1.1" )
      {
         QMessageBox::information(this,
                                  QObject::tr("QEVEN"),
                                  QObject::tr( "EL fichero no pertenece a una version compatible " ) );
         return;
      }
   }

   Reset();
   projectFileName = file.fileName();
   m_video_list->set_Format(root.attribute( "format" ) );
   Project_Selected(m_video_list->getFormat());

   QDomElement child = root.firstChildElement();

   while ( !child.isNull() )
   {
      parseVideoElement( child );
      child = child.nextSiblingElement();
   }

   Update_List();
   Update_Space_Bar();
   Enable_Buttons();
}

//HAY QUE PONER AQUI A CARGAR EL WIDHT Y HEIGHT ORIGINALES DEL VIDEO
// Y LOS ACTUALES
/**
  * Funcion para parsear cada uno de los elementos del proyecto salvado
  * @param element nodo de elementos a parsear
  */
void Main_Window::parseVideoElement(const QDomElement &element)
{
   QString path = element.attribute( "path" );
   QString filename = element.attribute( "filename" );
   int vbitrate = 0;
   int abitrate = 0;
   int duration = 0;
   QString aspect = "";
   bool has_sub = false;
   QString subsPath = "";
   QString subsFont = "";
   int subScale = 3;
   float fps = 29.997;
   int width = 720;
   int height = 480;
   int woriginal = 720;
   int horiginal = 480;
   int resolution = 0;
   int aid = 0;


   QDomElement item = element.firstChildElement();
   while ( !item.isNull() )
   {
      if( item.tagName() == "bitrate" )
      {
         vbitrate = item.attribute( "video" ).toInt();
         abitrate = item.attribute( "audio" ).toInt();
      }
      else
      {
         if( item.tagName() == "aspect" )
         {
            aspect = item.attribute( "value" );
         }
         else
         {
            if( item.tagName() == "duration" )
            {
               duration = item.attribute( "value" ).toInt();
            }
            else
            {
               if( item.tagName() == "subtitles" )
               {
                  has_sub = true;
                  subsPath = item.attribute( "path" );
                  subsFont = item.attribute( "font" );
                  subScale = item.attribute( "scale" ).toInt();
               }
               else
               {
                  if(item.tagName()== "resolution")
                  {
                     width = item.attribute("width").toInt();
                     height = item.attribute("height").toInt();
                     woriginal = item.attribute("woriginal").toInt();
                     horiginal = item.attribute("horiginal").toInt();
                     resolution = item.attribute("resolution").toInt();
                  }
                  else
                      if(item.tagName() == "fps")
                      {
                        fps = item.attribute("fps").toFloat();
                        if(fps >= 29 && fps < 30)
                           fps = 30000/1001;
                        }
                     else
                         if(item.tagName() == "aid")
                         {
                            aid = item.attribute("value").toInt();
                         }


               }
            }
         }
      }

      item = item.nextSiblingElement();
   }

   Video_Item* nuevo = new Video_Item( filename,
                                       path,
                                       vbitrate,
                                       abitrate,
                                       duration,
                                       m_video_list->getFormat(),
                                       aspect,
                                       has_sub,
                                       subsPath,
                                       subsFont,
                                       subScale );
   nuevo->set_Fps(fps);
   nuevo->set_Video_Resolution(resolution);
   nuevo->set_Video_Height(height);
   nuevo->set_Video_Height_Original(horiginal);
   nuevo->set_Video_Width(width);
   nuevo->set_Video_Width_Original(woriginal);
   nuevo->set_aid(aid);

   m_video_list->add_Video( nuevo );
}

void Main_Window::Show_Wizard()
{
   this->show();
   wizard_gui->show();
}


/**
  * Funcion para preguntar si se desea salvar o no
  */
bool Main_Window::maybeSave()
{
   //    si cierra el proyecto con la opcion del menu flagClose es true
   //    y se pregunta si se quiere salvar
   if ( modified && flagClose == true)
   {
      int ret = QMessageBox::warning( this,
                                      tr( "Advertencia" ),
                                      tr( "Desea guardar el proyecto antes de salir?" ),
                                      QMessageBox::Yes | QMessageBox::Default,
                                      QMessageBox::No,
                                      QMessageBox::Cancel | QMessageBox::Escape );

      if ( ret == QMessageBox::Yes )
      {
         SaveForce();

         return true;
      }
      else
      {
         if (ret == QMessageBox::Cancel)
         {
            // seteamos la opcion de flagClose en FALSE para evitar que siempre se quiera "salir"
            flagClose = false;
            return false;
         }
      }
   }

   return true;
}
/**
  * Funcion para redirigir el proceso de salvado
  */
void Main_Window::SaveForce()
{
   if( projectFileName != "" )
   {
      Save( projectFileName );
   }
   else
   {
      Save();
   }
}
/**
  * Funcion para cambiar las direccions globales de los ejecutables externos del mencoder y mplayer
  */
void Main_Window::SetProgramDir(QString MEncoder, QString MPlayer)
{
   this->MEncoder = MEncoder;
   this->MPlayer = MPlayer;
}



/*
  * Funcion para crear el icontray y manejarlo
  */
void Main_Window::createTrayIcon()
{
   trayIconMenu = new QMenu(this);
   trayIconMenu->addAction(this->exitProgram_action);

   //   agregar mas actions al menu del systray
   trayIcon = new QSystemTrayIcon(this);
   trayIcon->setContextMenu(trayIconMenu);
   trayIcon->setIcon(QIcon(QPixmap(":/main/Image/qeven_tray.png")));

   // adicionado un toolTip para la aplicacion
   trayIcon->setToolTip( tr("QEVEN  No hay trabajos pendientes"));

   // inicializar el flag para el primer uso
   this->firstTrayMsg = true;
}

void Main_Window::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
   switch (reason)
   {
   case QSystemTrayIcon::Trigger:  // creo q no hace falta la accion del doble click
      if(visible_Window==0)   // si se debe mostrar la ventana principal
      {
         if( true == this->isHidden() )   // si esta oculta la mostramos
         {
            this->Show_Main_Window();
         }
         else  // si esta visible la escondemos
         {
            this->close();
         }
      }
      else  // si se debe mostrar la ventana de conversion
      {
         if( true == this->converter_gui->isHidden() )   // si esta oculta la mostramos
         {
            this->converter_gui->show();
         }
         else  // si esta visible la escondemos
         {
            this->converter_gui->close();
         }
      }

      // this->show();
      break;
   default:
      ;
   }
}


/*
  * Funcion para copiar la fuente hacia el directorio de fuentes del proyecto
  */
void Main_Window::add_Font(QString fontPath)
{
#ifdef Q_OS_WIN32
   QSettings ini(QSettings::IniFormat,QSettings::UserScope,
                 QCoreApplication::organizationName(),
                 QCoreApplication::applicationName());
   QDir font(QFileInfo(ini.fileName()).absolutePath()+"/QEVEN/Fonts/");
   if(!font.exists())
   {
      font.mkdir(QFileInfo(ini.fileName()).absolutePath()+"/QEVEN/Fonts");
   }
   QStringList aux_name =  fontPath.split( "/" );
   QString fontname = aux_name.last();
   QFile::copy(fontPath,font.path() +"/"+ fontname);

#endif
#ifdef Q_OS_LINUX
   QDir font(QDir::homePath()+"/.QEVEN/Fonts/");
   if(!font.exists())
   {
      font.mkdir(QDir::homePath()+"/.QEVEN/Fonts");
   }
   QStringList aux_name =  fontPath.split( "/" );
   QString fontname = aux_name.last();
   QFile::copy(fontPath,font.path() +"/"+ fontname);
#endif

}

void Main_Window::Show_Main_Window()
{
   visible_Window = 0;

   // adicionado un toolTip para la aplicacion
   // volvemos a poner el estado en Sin trabajos....
   trayIcon->setToolTip(  tr("QEVEN  No hay trabajos pendientes") );

   this->show();
}


/****************  EVENTOS ****************/

/**
 * Evento. Funcion que se ejecuta al cerrar la ventana
 */
void Main_Window::closeEvent( QCloseEvent *event)
{
   if(maybeSave())
   {
      writeSettings();

      if(flagClose == true )
      {
         event->accept();
         qApp->quit();
      }
      else
      {
         this->hide();
         if( true == this->firstTrayMsg ) // solo lo mostramos si es la 1ra vez...
         {
            this->firstTrayMsg = false;   // adios al mensaje del tryIcon
            trayIcon->showMessage(tr("Info"), tr("La aplicacion se ha minimizado"));
         }

         // adicionado un toolTip para la aplicacion
         // si no hay trabajos pendientes
         if( 0 == this->visible_Window )
         {
            trayIcon->setToolTip(tr("QEVEN  No hay trabajos pendientes"));
         }

         event->ignore();
      }
   }
   else
      event->ignore();
}

/**
* Funcion para aceptar el drop,
* Toma los datos del evento drop y si su url pertenecen a un archivo de medias que puede ser importado
* los adiciona a la biblioteca.
* @param event
*/
void Main_Window::dropEvent( QDropEvent *event )
{
   m_video_list->setProgramDir(MPlayer);
   if( event->mimeData()->hasUrls() )
   {
      for( int i=0; i < event->mimeData()->urls().size(); i++)
      {
         //if(i>0) scrubForward();
         //Adicionados extensiones ogg y ogv. 2008-11-18 12:36:57
         QUrl url = event->mimeData()->urls()[i];
         QString filePath = url.toLocalFile();
         if( filePath.endsWith( ".mpg", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".mpeg", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".ogg", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".ogv", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".avi", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".wmv", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".divx", Qt::CaseInsensitive) ||
             filePath.endsWith( ".vob", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".mp4", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".xvid", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".asf", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".ogm", Qt::CaseInsensitive) ||
             filePath.endsWith( ".mov", Qt::CaseInsensitive) ||
             filePath.endsWith( ".rmvb", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".rm", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".flv", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".mkv", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".mka", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".qt", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".DAT", Qt::CaseInsensitive ) ||
             filePath.endsWith( ".m2t", Qt::CaseInsensitive ))
         {
            m_video_list->add_Video( filePath );

            Update_Space_Bar();
            Update_List();
            modified = true;
         }
      }
   }
}

/**
* Funcion para aceptar el drag dentro de la aplicacion
* @param event
*/
void Main_Window::dragEnterEvent(QDragEnterEvent *event)
{
   //if ( event->mimeData()->hasFormat( "text/plain " ) )
   event->acceptProposedAction();
}

void Main_Window::keyReleaseEvent(QKeyEvent * event)
{
   if(listWidget->hasFocus() && event->key() == Qt::Key_Delete )
      this->Remove_Video();
   else
      event->ignore();
}


